(* ==================================================================
The use of this Package is described in "Economic and Financial
Modeling with Mathematica", Hal Varian, editor, TELOS/Springer-Verlag,
1993, ISBN 0-387-97882-8.  $44.95 (Includes 1.44 MS-DOS disk.)  To
place order call 1-800-777-4643 or fax 1-201-348-4505.
=====================================================================*)

(*^

::[paletteColors = 128; currentKernel; 
	fontset = title, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeTitle, center, M7, bold, e8,  24, "Times"; ;
	fontset = subtitle, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeTitle, center, M7, bold, e6,  18, "Times"; ;
	fontset = subsubtitle, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeTitle, center, M7, italic, e6,  14, "Times"; ;
	fontset = section, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeSection, grayBox, M22, bold, a20,  18, "Times"; ;
	fontset = subsection, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeSection, blackBox, M19, bold, a15,  14, "Times"; ;
	fontset = subsubsection, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeSection, whiteBox, M18, bold, a12,  12, "Times"; ;
	fontset = text, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12;
	fontset = smalltext, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  10, "Times"; ;
	fontset = input, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeInput, M42, N23, bold, L-5,  12, "Courier"; ;
	fontset = output, output, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23, L-5,  12, "Courier"; ;
	fontset = message, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23, R65535, L-5,  12, "Courier"; ;
	fontset = print, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23, L-5,  12, "Courier"; ;
	fontset = info, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23, B65535, L-5,  12, "Courier"; ;
	fontset = postscript, PostScript, formatAsPostScript, output, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeGraphics, M7, l34, w282, h287,  12, "Courier"; ;
	fontset = name, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, italic, B65535,  10, "Times"; ;
	fontset = header, inactive, noKeepOnOnePage, preserveAspect, M7,  12;
	fontset = Left Header, nohscroll, cellOutline,  12, "Palatino"; ;
	fontset = footer, inactive, noKeepOnOnePage, preserveAspect, center, M7,  12;
	fontset = Left Footer, cellOutline, blackBox,  12, "Palatino"; ;
	fontset = help, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  10, "Times"; ;
	fontset = clipboard, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12;
	fontset = completions, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12;
	fontset = special1, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12;
	fontset = special2, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12;
	fontset = special3, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12;
	fontset = special4, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12;
	fontset = special5, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12;
	next21StandardFontEncoding; ]
:[font = input; initialization; preserveAspect; ]
*)
(* BeginPackage["Arima`"]; *)

Arima::usage = 
	"The Arima package provides a unified set of tools
	for manipulating autoregressive, moving average models.";
	
(*
:[font = input; preserveAspect; ]
(* Use f[x_?Q] form to call a function to check args.*)
:[font = text; inactive; initialization; preserveAspect; ]
Modification Notes

9/10/92...Watch those minus signs.
6/18/92...Package wrapper added and partially tested.
6/17/92...Document functions with usage messages.
6/16/92...Killed bug in ToCanonicalForm for moving average models.
6/16/92...New ToCosPoly no longer needs <<Trigonometry` to simplify cosine polynomials.

:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Usage Messages
:[font = input; preserveAspect; ]
(* SYMBOL LIST *)

ArimaSymbolList::usage =
	"ArimaSymbolList is a global variable that holds the
	set of symbols which have been bound to ARIMA models.";

ArimaResetSymbolList::usage =
	"ArimaResetSymbolList[] sets ArimaSymbolList to {}"; 

ArimaPrintSymbolList::usage =
	"ArimaPrintSymbolList[] prints a summary of the models
	in the ArimaSymbolList."

:[font = input; initialization; preserveAspect; startGroup; ]
*)
(* DATA STRUCTURES *)

ArimaTransferFunction::usage =
	"ArimaTransferFunction[y_ARIMA] returns the transfer
	function of the arima model y as a rational polynomial
	of the protected variable z.";

ArimaPhi::usage =
	"ArimaPhi[y_ARIMA] returns a list of autoregressive
	coefficients from the model, nomalized so that the
	leading coefficient is 1.";
	
ArimaTheta::usage = 
	"ArimaTheta[Y_ARIMA] returns a list of the moving
	average coefficients of the model, normalized so that
	the leading coefficient is 1.";

ArimaP::usage = 
	"ArimaP[y_ARIMA] returns autoregression order.";
	
ArimaQ::usage = 
	"ArimaQ[y_ARIMA] returns moving average order.";

ArimaD::usage = 
	"ArimaD[y_ARIMA] returns the order of differencing.";

ArimaOrder::usage = 
	"ArimaOrder[y_ARIMA] returns the triple {p,d,q}.";

ArimaIsMAQ::usage = 
	"ArimaIsMAQ[y_ARIMA] returns True if the process is
	pure moving average, and False otherwise.";
	
ArimaIsArQ::usage = 
	"ArimaIsArQ[y_ARIMA] returns True if the process is
	pure autoregression, and False otherwise.";

ArimaIsArmaQ::usage = 
	"ArimaIsArmaQ[y_ARIMA] returns True if the process is
	pure ARMA with no differencing.";

ArimaMean::usage = 
	"ArimaMean[y_ARIMA] returns the mean of the process.";

ArimaConstant::usage = 
	"ArimaConstant[y_ARIMA] returns the constant term 
	associated with the process.";
	
ArimaScale::usage = 
	"ArimaScale[y_ARIMA] returns the scale of the white
	noise input to the ARIMA model.";
	
ArimaLabel::usage = 
	"ArimaLabel[y_ARIMA] returns the text string
	associated with symbolic name of the process. This
	string is used in printing the process.";
	 
ArimaError::usage = 
	"ArimaError[y_ARIMA] returns the white noise process
	which is the input to the model specified by y.";
	
ArimaErrorLabel::usage =
	"ArimaErrorLabel[y_ARIMA] returns the text string
	associated with the symbolic name of the error 
	input process.  This string appears in printing.";

(*
:[font = message; inactive; preserveAspect; ]
General::spell1: 
   Possible spelling error: new symbol name "ArimaP"
     is similar to existing symbol "Arima".
:[font = message; inactive; preserveAspect; ]
General::spell: 
   Possible spelling error: new symbol name "ArimaQ"
     is similar to existing symbols {Arima, ArimaP}.
:[font = message; inactive; preserveAspect; endGroup; ]
General::spell: 
   Possible spelling error: new symbol name "ArimaD"
     is similar to existing symbols {Arima, ArimaP, ArimaQ}.
:[font = input; initialization; preserveAspect; startGroup; ]
*)
(*  CREATION  *)

ArimaCreate::usage = 
	"ArimaCreate[sym_Symbol, input_ARIMA,phi_List, d_,
	 theta_List, mean_:0] builds an ARIMA model with the
	 listed parameters.";
	 
DefineWhiteNoise::usage = 
	"DefineWhiteNoise[sym_Symbol, scale_:1] defines a white
	noise ARIMA model having the input scale parameter.";

DefineAr::usage = 
	"DefineAr[sym_Symbol, input_ARIMA, phi_List, mean_:0]
	builds an autoregression with input parameters. Note that
	the scale is taken from that of the input white noise.";
					
DefineMA::usage = 
	"DefineMA[sym_Symbol, input_ARIMA, theta_List, mean_:0]
	builds a moving average with input parameters. Note that
	the scale is taken from that of the input white noise.";
	
DefineArma::usage = 
	"DefineArma[sym_Symbol, input_ARIMA, phi_List, theta_List, mean_:0]
	builds an ARMA model with input parameters. Note that
	the scale is taken from that of the input white noise.";
	
DefineArima::usage = 
	"DefineArima[sym_Symbol, input_ARIMA, phi_List, d_,
	 theta_List, mean_:0] builds an ARIMA model from input
	 parameters. Note that the scale is taken from that
	 of the input white noise.";
	 
ArimaRule::usage = 
	"ArimaRule[y_ARIMA, process_Symbol, error_Symbol] returns 
	a substitution rule associated with the input Arima model
	using the input symbols for the process and error.";
	
(*
:[font = message; inactive; preserveAspect; endGroup; ]
General::spell1: 
   Possible spelling error: new symbol name "DefineArima"
     is similar to existing symbol "DefineArma".
:[font = input; initialization; preserveAspect; ]
*)
(*  BASIC OPERATIONS  *)

ArimaDifference::usage = 
	"ArimaDifference[new_Symbol, y_ARIMA, d_:1] defines
	a new ARIMA process which results from applying
	the chosen order of differencing to the input process.
	The input process is NOT affected.";
	
ArimaPrint::usage = 
	"ArimaPrint[y_ARIMA] prints the model in standard
	form.";

(*
:[font = input; initialization; preserveAspect; ]
*)
(*  BACKSHIFT POLYNOMIALS  *)

ArimaPhiPolynomial::usage = 
	"ArimaPhiPolynomial[y_ARIMA] returns the denominator
	of the transfer function of y as a polynomial in the
	protected symbol z.";
	
ArimaThetaPolynomial::usage = 
	"ArimaThetaPolynomial[y_ARIMA] returns the numerator
	of the transfer function of y as a polynomial in the
	protected symbol z.";

ArimaZeros::usage = 
	"ArimaZeros[y_ARIMA] returns a list of the zeros of 
	the	transfer function.  These are also the zeros of
	the theta polynomial.";
	
ArimaPoles::usage = 
	"ArimaPoles[y_ARIMA] returns a list of the poles of
	the transfer function.  These are also the zeros of
	the	phi polynomial.";
	
ArimaIsStationaryQ::usage = 
	"ArimaIsStationary[y_ARIMA] returns True if the poles
	of the transfer function lie outside of the unit circle.";

ArimaIsInvertibleQ::usage = 
	"ArimaIsInvertibleQ[y_ARIMA] returns True if the zeros
	of the transfer function lie outside of the unit circle.";
	
ArimaPlotZeros::usage =
	"ArimaPlotZeros[y_ARIMA] returns a plot of the reciprocals
	of the zeros of the transfer function.  Reciprocals hopefully
	lie inside the unit circle.";
	
ArimaPlotPoles::usage = 
	"ArimaPlotPoles[y_ARIMA] returns a plot of the reciprocals
	of the poles of the transfer function.  Reciprocals hopefully
	lie inside the unit circle.";

	
(*
:[font = input; initialization; preserveAspect; ]
*)
(*  SPECTRAL  DENSITY  *)

ArimaSDF::usage = 
	"ArimaSDF[y_ARIMA] returns a pure function that
	evaulates as the spectral density function of the
	process.";
	
ArimaPlotSDF::usage = 
	"ArimaPlotSDF[y_ARIMA, options___] returns a plot of
	the spectral density, passing the input options to
	the call to Plot.";

(*
:[font = input; initialization; preserveAspect; ]
*)
(*  COVARIANCE AND CORRELATION  *)

ArimaCovariance::usage = 
	"ArimaCovariance[y_ARIMA, n_Integer:5] returns a
	list of n covariances {g[0], g[1], ... g[n-1]},
	beginning with the variance of the process.";

ArimaCovarianceMatrix::usage = 
	"ArimaCovarianceMatrix[y_ARIMA, dim_Integer] returns
	the dim-by-dim covariance matrix of the process.";

ArimaCorrelation::usage = 
	"ArimaCorrelation[y_ARIMA, n_Integer:5] returns the
	list of n correlations {1, r[1], r[2], ..., r[n-1]}
	associated with the input process.";


(*
:[font = input; initialization; preserveAspect; ]
*)
(*  BIAS  *)

ArimaBiasMatrix::usage = 
	"ArimaBiasMatrix[p_, trend_] returns the matrix
	B[p] associated with the bias of least squares 
	estimators of an AR(p) model with trend estimated
	trend parameters.";
	
ArimaCoefficientBias::usage = 
	"ArimaCoefficientBias[ y_ARIMA, trend_ ] returns the
	list of the order 1/T bias terms associated with
	least squares estimators of the coefficients of an
	Ar model with trend estimated trend parameters.";


(*
:[font = input; initialization; preserveAspect; ]
*)
(*  PREDICTION MSE  *)

ArimaMAWeights::usage = 
	"ArimaMAWeights[y_ARIMA, n_Integer:10] returns a
	list of the first n coefficients of the moving average
	representation of the process y.";
	
ArimaMSE::usage = 
	"ArimaMSE[y_ARIMA, n_Integer:10] returns a list of
	the optimal prediction mean squared error. An alternative
	form ArimaMSE[y_ARIMA, f_Integer:10, nObs_] returns
	the list of mean squared error associated with estimated
	predictors.";


(*
:[font = input; initialization; preserveAspect; endGroup; ]
*)
(*  REALIZATIONS  *)

ArimaNRealize::usage = 
	"ArimaNRealize[y_ARIMA, n_Integer:25, opts___] := \
	generates a Gaussian time series of length n which
	is a partial realization of the input model y.";

(*
:[font = subsubsection; inactive; initialization; preserveAspect; ]
Package Messages
:[font = input; initialization; preserveAspect; ]
*)
Arima::rule = "Rule `1` is not properly formed.";

Arima::noise = "Symbol `1` is not white noise.";

(*
:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Special Global Symbols
:[font = input; initialization; preserveAspect; startGroup; ]
*)
Unprotect[z, t, B];

	z::usage = "Reserved for transfer function power."
	t::usage = "Reserved for time index."
	B::usage = "Reserved for back-shift lag operator."

Clear[z, t, B];
Protect[z, t, B];

(*
:[font = output; output; inactive; preserveAspect; ]
"Reserved for transfer function power."
;[o]
Reserved for transfer function power.
:[font = output; output; inactive; preserveAspect; ]
"Reserved for time index."
;[o]
Reserved for time index.
:[font = output; output; inactive; preserveAspect; endGroup; endGroup; ]
"Reserved for back-shift lag operator."
;[o]
Reserved for back-shift lag operator.
:[font = subsubsection; inactive; Cclosed; preserveAspect; startGroup; ]
Some Extra Global Utilities
:[font = input; initialization; preserveAspect; endGroup; ]
*)
DoubleListPlot[list1_List,list2_List,extra_:{} ] := Module \
	[	{lp1, lp2},
		lp1 = ListPlot[list1, PlotJoined->True, 
					DisplayFunction->Identity];
		lp2 = ListPlot[list2, PlotJoined->True,
					PlotStyle->GrayLevel[0.5],
					DisplayFunction->Identity];
		Show[ Graphics[extra],lp1,lp2,
				DisplayFunction->$DisplayFunction]
	]
	

(*
:[font = input; initialization; preserveAspect; ]
*)
(* Begin["`Private`"]; *)

(*
:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Polynomial utilities,  type checking, and special protected symbols
:[font = text; inactive; initialization; preserveAspect; ]
Notice that PolyZeros functions explicitly in the protected symbol z.  Not only does this save identifying the symbol on the fly, but also avoids problems when the coefficients themselves are
symbolic.

The protected symbols are z (power in internal polynomials), B (backshift function), and t (generic time index).
:[font = input; initialization; preserveAspect; ]
*)
Clear[
	ScalarQ, PosIntegerQ,
	CoefToPoly, PolyZeros	]
(*
:[font = input; initialization; preserveAspect; ]
*)
ScalarQ[x_Real] 	:= True;
ScalarQ[x_Integer]	:= True;
ScalarQ[x_Symbol]	:= True;
ScalarQ[_] := False;

PosIntegerQ[x_Integer] := True /; x > 0;
PosIntegerQ[_] := False;
(*
:[font = input; initialization; preserveAspect; ]
*)
CoefToPoly[c_List,x_] := Module	\
	[	{i},
		Plus @@ (c Table[x^i,{i,0,Length[c]-1}])
	];

PolyZeros[1,x_Symbol]  := {} 
PolyZeros[p_,x_Symbol] := {} /; Not[MemberQ[Variables[p],x]]
PolyZeros[p_,x_Symbol] := x /. Solve[p == 0,x];
(*
:[font = input; initialization; preserveAspect; startGroup; ]
*)
PolyZeros[1+z+z^2,z]
(*
:[font = output; output; inactive; initialization; preserveAspect; endGroup; ]
{(-1 + I*3^(1/2))/2, (-1 - I*3^(1/2))/2}
;[o]
 -1 + I Sqrt[3]  -1 - I Sqrt[3]
{--------------, --------------}
       2               2
:[font = input; initialization; preserveAspect; startGroup; ]
*)
ScalarQ[r4]
CoefToPoly[{1,2,3},z]
PolyZeros[(1-z)(1-.5z),z]
(*
:[font = output; output; inactive; initialization; preserveAspect; ]
True
;[o]
True
:[font = output; output; inactive; initialization; preserveAspect; ]
1 + 2*z + 3*z^2
;[o]
             2
1 + 2 z + 3 z
:[font = output; output; inactive; initialization; preserveAspect; endGroup; endGroup; ]
{1, 2.}
;[o]
{1, 2.}
:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Data Structures
:[font = text; inactive; initialization; preserveAspect; ]
Comments:

Functions in this block are permitted to use the specific list representation of an ARIMA model.

:[font = text; inactive; preserveAspect; ]
By definition, the Mathematica representation of ARIMA objects has the following six element canonical list form:
                       { T(z)/P(z), d,   mean, scale,   label, error label } ,
                       
and the head of the list is ARIMA.  The ratio T(z)/P(z) is the transfer function and d is the order of differencing, where it is assumed (and later enforced) that none of the zeros of P(z) are equal to 1.  Following the usual convention, such zeros are incorporated into the difference value d.


:[font = input; initialization; preserveAspect; ]
*)
Clear[  ARIMA, ArimaTransferFunction,
		ArimaPhi, ArimaD, ArimaTheta,
		ArimaP, ArimaD, ArimaQ, ArimaOrder, 
		ArimaIsMAQ, ArimaIsArQ, ArimaIsArmaQ,
		ArimaMean, ArimaConst, ArimaScale, WhiteNoiseScale,
		ArimaLabel, ArimaError, ArimaErrorLabel]

ArimaTransferFunction[ARIMA[tf_,___]] := tf;

ArimaPhi[ARIMA[tf_,___]]:=CoefficientList[Denominator[tf],z];
ArimaTheta[ARIMA[tf_,___]]:=CoefficientList[Numerator[tf],z];

ArimaP[ARIMA[tf_,___]]:= Exponent[Denominator[tf],z];
ArimaQ[ARIMA[tf_,___]]:= Exponent[Numerator[tf],z];
ArimaD[ARIMA[_,d_,___]]:= d;

ArimaOrder[ARIMA[tf_,d_,___]] := \
				{Exponent[Denominator[tf],z],
					d, Exponent[Numerator[tf],z]}

ArimaIsMAQ[y_ARIMA] := ArimaP[y]==0;
ArimaIsArQ[y_ARIMA] := ArimaQ[y]==0;

ArimaIsArmaQ[ARIMA[_,0,___]]:= True;
ArimaIsArmaQ[_]	:= False;

ArimaMean [ARIMA[_,_,m_,___]]	:= m;
ArimaConstant[y_ARIMA] := ArimaMean[y]/(Plus@@ArimaPhi[y]);

ArimaScale[ARIMA[_,_,_,s_,___]] := s;

ArimaLabel	[ARIMA[_,_,_,_,l_,___]] := l;

ArimaError      [ARIMA[_,_,_,_,_,e_]] := ToExpression[e];
ArimaErrorLabel	[ARIMA[_,_,_,_,_,e_]] := e;
(*
:[font = input; initialization; preserveAspect; startGroup; ]
*)
Clear[ArimaRule]

ArimaRule [ y_ARIMA, ySym_Symbol, eSym_Symbol] := Module \
	[	{phi, p, d, dSym, theta, q, j},
		d = ArimaD[y];
		If[ d==0, dSym = 1, dSym = (1-B)^d];
		phi  = -Drop[ArimaPhi[y],1]		; p = Length[phi];
		theta = Drop[ArimaTheta[y],1]	; q = Length[theta];
		Rule[dSym ySym[t],
			phi . Table[dSym ySym[t-j],{j,p}]
			+ eSym[t] + theta . Table[eSym[t-j],{j,q}]]
	]
(*
:[font = message; inactive; preserveAspect; endGroup; ]
General::spell: 
   Possible spelling error: new symbol name "dSym"
     is similar to existing symbols {eSym, ySym}.
:[font = input; initialization; preserveAspect; ]
*)
Clear[
	ArimaSymbolList, ArimaResetSymbolList,
	ArimaPrintSymbolList ];

ArimaSymbolList := {};

ArimaResetSymbolList[] := ArimaSymbolList = {};

ArimaPrintSymbolList[] :=
	StringJoin[ArimaLabel[#],ToString[ArimaOrder[#]]] & /@ ArimaSymbolList;


(*
:[font = input; initialization; preserveAspect; endGroup; ]
*)
Clear[DefineWhiteNoise, ArimaIsWhiteNoiseQ]

DefineWhiteNoise[sym_Symbol, scale_:1] :=	With \
	[	{old = ArimaSymbolList},
		ArimaSymbolList = Union[old, {sym}];
		Set[sym, ARIMA[1,0,0,scale, ToString[sym], "WN"]];
	];

ArimaIsWhiteNoiseQ[ARIMA[1,0,0,___]] := True;
ArimaIsWhiteNoiseQ[_] := False;

(*
:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Creators
:[font = text; inactive; initialization; preserveAspect; ]
Comments:

Code for creating internally managed data structures.

Two formats describe the input ARIMA model.  The most direct is to use the coefficients of the process specified in the lists phi and theta.  Alternatively, one can supply a rule to DefineArma.

All calls to build ARIMA models eventually operate through the CreateArima function, which adds the built model to an internal table of processes.

:[font = input; initialization; preserveAspect; ]
*)
Clear[ToCanonicalForm];
	
ToCanonicalForm [phi_List, d_, theta_List] := Module \
	[	{poly, num, den},
		num = CoefToPoly[Join[{1},-theta],z];
		den = CoefToPoly[Join[{1},-phi],z];
		(* Print["phi ", phi, "theta ", theta]; *)
		ToCanonicalForm[num/den, d]		(* multiple entry *)
	];
	
ToCanonicalForm[rationalPoly_,d_] := Module	\
	[	{poly , zeros, newD, foundD, const, norm},
		poly = Cancel[rationalPoly];
		If[False,
			zeros = PolyZeros[Denominator[poly],z];
			newD = d;
			foundD = Count[1+Chop[zeros-1],1];
			If[foundD!=0,
				newD = foundD + newD;
				poly = Cancel[poly*(1-z)^foundD]],
			newD = d;
			];
		const = Coefficient[Numerator[poly],z,0];
		norm[p_] := Expand[(1/const) p];  (* local function *)
		If[const!=1,
		  poly = norm[Numerator[poly]]/norm[Denominator[poly]]];
		{poly,newD}
	];

(*
:[font = input; initialization; preserveAspect; startGroup; ]
*)
Clear[ArimaCreate]

ArimaCreate[sym_Symbol, input_?ArimaIsWhiteNoiseQ,
			phi_List, d_, theta_List, mean_:0] := Module \
	[	{old = ArimaSymbolList, trans},
		ArimaSymbolList = Union[old, {sym}];
		trans = ToCanonicalForm[phi,d,theta];
		Set[sym, ARIMA[trans[[1]],trans[[2]],
				mean, ArimaScale[input],
				ToString[sym],ArimaLabel[input]]];
	];
	
ArimaCreate[sym_Symbol, input_ARIMA,
			phi_List, d_, theta_List, mean_:0] := Module \
	[	{inputTrans, inputD, tp, trans, D},
		inputTrans = ArimaTransferFunction[input];
		inputD     = ArimaD[input];
		tp = ToCanonicalForm[phi,d,theta];
		trans = Together[ ExpandAll[ tp[[1]] inputTrans ]];
		D = tp[[2]] + inputD;
		ArimaCreate[sym, ToExpression[ArimaErrorLabel[input]],
			CoefficientList[Denominator[trans],z], D,
			CoefficientList[Numerator[trans],z], mean] (* ?? *)
	]
	
(*
:[font = message; inactive; preserveAspect; endGroup; ]
General::spell1: 
   Possible spelling error: new symbol name "inputD"
     is similar to existing symbol "input".
:[font = input; initialization; preserveAspect; endGroup; ]
*)
Clear[DefineAr, DefineMA, DefineArma, DefineArima];


DefineAr[sym_Symbol, input_ARIMA, phi_List, mean_:0] :=
	If[ArimaIsWhiteNoiseQ[input],
		ArimaCreate[sym, input, phi,0,{},  mean],
		Message[Arima::noise,input]]
	
DefineMA[sym_Symbol, input_ARIMA, theta_List, mean_:0] :=
	If[ArimaIsWhiteNoiseQ[input],
		ArimaCreate[sym, input, {},0,theta,  mean],
		Message[Arima::noise,input]]
	
DefineArma[sym_Symbol, input_ARIMA, phi_List, theta_List, mean_:0] :=
	If[ArimaIsWhiteNoiseQ[input],
		ArimaCreate[sym, input, phi,0,theta,mean],
		Message[Arima::noise,input]]
		
DefineArima[sym_Symbol, input_ARIMA, phi_List, d_, theta_List, mean_:0] :=
	If[ArimaIsWhiteNoiseQ[input],
		ArimaCreate[sym, input, phi,d,theta,mean],
		Message[Arima::noise,input]]
	
(*
:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Basic operations: Differencing
:[font = input; initialization; preserveAspect; endGroup; ]
*)
Clear[ArimaDifference]

ArimaDifference[new_Symbol, y_ARIMA, d_:1] :=
	CreateArima[new, ArimaError[y],
				ArimaPhi[y], d+ArimaD[y], ArimaTheta[y],
				ArimaMean[y]]
(*
:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Basic output:  Printing an ARIMA model
:[font = text; inactive; initialization; preserveAspect; ]
Comments:

Printing seems rather complex due to all of the variations on ARIMA models.  Handled here through a special set of rules that manage the cases.

:[font = input; initialization; preserveAspect; endGroup; ]
*)
Clear[ArimaPrint]

Format[ ARIMA[_,_,_,_,n_,_][t_] ] := Subscripted[n[t]];

ArimaPrint[y_ARIMA] :=	\
	Print[ StringJoin[ArimaLabel[y],
					  "[t] ~ ",
					  ArimaErrorLabel[y],
					  "[",ToString[ArimaScale[y]],"]"  ]
	] /; ArimaIsWhiteNoiseQ[y];
	
ArimaPrint[y_ARIMA] := Module	\
	[	{ySym, phi, p, d, dSym, theta, q, j},
		ySym = ToExpression[ ArimaLabel[y] ];
		d = ArimaD[y];
		If[ d==0, dSym = 1, dSym = (1-B)^d];
		eSym = ToExpression[ ArimaErrorLabel[y] ];
		phi = -Drop[ArimaPhi[y],1]		; p = Length[phi];
		theta = Drop[ArimaTheta[y],1]	; q = Length[theta];
		SequenceForm[dSym ySym[t]," = "
			+ phi.Table[dSym ySym[t-j],{j,p}]
			+ eSym[t] + theta . Table[eSym[t-j],{j,q}]]
	]
(*
:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Backshift Polynomial Forms
:[font = input; initialization; preserveAspect; ]
*)
Clear[ArimaPhiPolynomial, ArimaThetaPolynomial,
		ArimaZeros, ArimaPoles,
		ArimaIsStationaryQ, ArimaIsInvertibleQ]
	
ArimaPhiPolynomial[ARIMA[tf_,___]] := Denominator[tf];
ArimaThetaPolynomial[ARIMA[tf_,___]]:= Numerator[tf];

ArimaZeros[y_ARIMA] := PolyZeros[ArimaThetaPolynomial[y],z];
ArimaPoles[y_ARIMA] := PolyZeros[ArimaPhiPolynomial[y],z];

ArimaIsStationaryQ[ARIMA[_,d_,___]] := False /; d > 0;
ArimaIsStationaryQ[y_ARIMA] := 1 < Min[Abs[ArimaPoles[y]]];

ArimaIsInvertibleQ[y_ARIMA] := True /; 1<Min[Abs[ArimaZeros[y]]];

(*
:[font = input; initialization; preserveAspect; endGroup; ]
*)
Clear[ ArimaPlotZeros ];

Clear[ plotZeros ];
	plotZeros[{}] := {};
	plotZeros[z_] := Module[ {lp},
						lp = ListPlot[Transpose[{Re[z],Im[z]}],
										DisplayFunction->Identity];
						Print[z];
						Show[lp, Graphics[Circle[{0,0},1]],
									AspectRatio -> Automatic,
									DisplayFunction-> $DisplayFunction]
					];
	
ArimaPlotZeros[y_ARIMA] := plotZeros[ 1/ArimaZeros[y] ];
ArimaPlotPoles[y_ARIMA] := plotZeros[ 1/ArimaPoles[y] ];

(*
:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Spectral density function     
:[font = text; inactive; initialization; preserveAspect; ]
Comments:

Special issues involve pattern matching, namely making sure that the SDF of an ARIMA is indeed explicitly computed as a real-valued function.

SDF returns a function, which can be evaluated, mapped, and plotted just like any other function.
:[font = input; initialization; preserveAspect; endGroup; ]
*)
Clear[ArimaSDF, toCosPoly];

							(* from f(z) to f(w) *)
toCosPoly[poly_] := Module \
	[	{coefs,i,j,p,c},
		coefs = CoefficientList[poly,z];
		p = Length[coefs];
		c = Table[
			2 Sum[coefs[[i]] coefs[[i+j]],{i,1,p-j}],
			{j,0,p-1}];
		c[[1]] = c[[1]] / 2;
		(CoefToPoly[c,z]) /. z^k_.->Cos[k z]
	]
		
ArimaSDF[y_ARIMA] := Module	\
	[	{num,den, s2, theta},
		s2 = (ArimaScale[y]^2)/(2 Pi);
		num = toCosPoly[ArimaThetaPolynomial[y]];
		den = toCosPoly[ArimaPhiPolynomial[y]];
		Function[theta,
			Evaluate[s2 num/den /. z->theta]
		]
	];

ArimaPlotSDF[y_ARIMA, options___Rule] := Module \
	[	{sdf,x},
		sdf = ArimaSDF[y];
		Plot[sdf[x], {x,0,Pi}, options]
	]
		
(*
:[font = subsubsection; inactive; initialization; preserveAspect; startGroup; ]
Covariance Functions     ..........   NEED differencing checks 
:[font = input; initialization; preserveAspect; startGroup; ]
*)
Clear [ ArimaCovariance, ArimaCorrelation,
		ArimaCovarianceMatrix, 
		covRecur  ];

ArimaCovariance[y_ARIMA, n_Integer:5] :=	Module	\
	[	{errorVar},
		errorVar = ArimaScale[y]^2;
		Join[{errorVar},Table[0,{n-1}]]
	]	/;	ArimaIsWhiteNoiseQ[y];


covRecur[phi_List, cov_List, n_Integer] := Module	\
	[	{i,p, soFar, result, coef},		(* walk out to length n *)
		coef = Reverse[- Drop[phi,1]]; p = Length[coef];
		soFar  = Length[cov];
		result = Join[cov, Table[0,{n-soFar}]];
		For[i=soFar+1, i<=n, i++,
			result[[i]] = coef . Take[result,{i-p,i-1}]];
		result
	];
	
ArimaCovariance[y_ARIMA, n_Integer:5] :=	Module	\
	[	{cov, phi,Ryy,Rye,c,rules,  value,j,i},
		cov[k_Integer] := cov[-k] /; k<0;	(* local function *)
		phi = ArimaPhi[y];
		p  = Length[phi] - 1;
		(* Print["AR(",p,") model"]; *)
		Ryy = Table[cov[i-j],{i,0,p},{j,0,p}];
		Rye = Table[0,{i,0,p}]; Rye[[1]]=1;
		c  = Table[cov[j],{j,0,p}];			(* must compute p+1 *)
		(* Print ["System:", Ryy.coef, "=", Rye]; *)
		rules = First[Solve[Ryy.phi == Rye, c]];
		value = (ArimaScale[y]^2) c/.rules;
		If[ n <= p+1,
			Take[value,n],
			covRecur[phi,value,n]]
	]	/;	ArimaIsArQ[y];
				
ArimaCovariance[y_ARIMA, n_Integer:5] :=	Module	\
	[	{c, q, value,j},
		c = ArimaTheta[y]; q = Length[c]-1;
		(* Print["MA(",q,") model"]; *)
		value=(ArimaScale[y]^2) Map[Sum[c[[j]]*c[[j+#]],{j,1,q+1-#}]&,
											Range[0,Min[n-1,q]]];
		If[n-1 <= q,
			value,
			Join[value,Table[0,{n-q+1}]]]
	]	/;	ArimaIsMAQ[y];
														(* BUG *)
ArimaCovariance[y_ARIMA, want_Integer:5] :=	Module	\
	[	{ar, ma, p,Ryy,Rye,c,rules, arLong,maLong,maWts, cov,k,n},
		cov[k_] := cov[-k] /; k<0;
		ar = ArimaPhi[y];	ma = ArimaTheta[y];
		p  = Length[ar]-1;	q = Length[ma]-1;
		If[want<(2+Max[p,q]), n=2+Max[p,q], n=want];
		(* Print["ARMA(",p,",",q,") model"]; *)
		arLong = Join[ar,Table[0,{i,1,n-(p+1)}] ];
		maLong = Join[ma,Table[0,{i,1,n-(q+1)}] ];
		(* Print["arlong", arLong, "maLong", maLong]; *)
		Ryy = Table[cov[i-j],{i,0,n-1},{j,0,n-1}];
		maWts = ArimaMAWeights[y,n-1];
		Rye = Map[Sum[maWts[[j]]*maLong[[j+#-1]],{j,1,n-#}]&, Range[1,n] ];
		c  = Table[cov[j],{j,0,n-1}];
		(* Print ["System:"]; Print [ Ryy.arLong, "=", Rye]; *)
		rules = First[Solve[Ryy.arLong==Rye, c]];
		c = c/.rules;
		(ArimaScale[y]^2) Together /@ c
	]	/; ArimaIsArmaQ[y];


ArimaCorrelation[y_ARIMA, n_Integer:5] :=
		With[{g = ArimaCovariance[y,n]},  Map[#/g[[1]]&,g] ];

	
ArimaCovarianceMatrix[y_ARIMA, dim_Integer] := Module	\
	[	{cov, i, j},
		cov = ArimaCovariance[y,dim];  (* only need n-1 *)
		Table[cov[[ 1+Abs[i-j] ]], {i,dim}, {j,dim}]
	];

(*
:[font = message; inactive; preserveAspect; endGroup; endGroup; ]
General::spell1: 
   Possible spelling error: new symbol name "coef"
     is similar to existing symbol "coefs".
:[font = subsubsection; inactive; initialization; preserveAspect; startGroup; ]
Bias in AR models
:[font = text; inactive; initialization; preserveAspect; ]
Comments:

References:
						 Shaman and Stine (1988), J. Amer. Statis. Assoc.
						 Stine and Shaman (1989).  Annals of Statistics

:[font = input; initialization; preserveAspect; ]
*)
Clear[b2Element, b3Element, b2, b3]

b2Element[i_,j_,p_]:=-1 /; (j<i<=p-j && 0==Mod[(i-j),2])
b2Element[i_,j_,p_]:= 1 /; (p-j<i<=j && 0==Mod[(i-j),2])						
b2Element[i_,j_,p_]:= 0  ;

b3Element[i_,j_,p_] := -1  /; 		(j <i<= p-j)
b3Element[i_,j_,p_] :=  1  /;		(p-j <i<= j)						
b3Element[i_,j_,p_] :=  0   ;

b2[p_] := Module	\
	[	{i,j},
		Table[b2Element[i,j,p],{i,0,p},{j,0,p}]
	]
	
b3[p_] := Module	\
	[	{i,j},
		Table[b3Element[i,j,p],{i,0,p},{j,0,p}]
	]
(*
:[font = input; initialization; preserveAspect; endGroup; ]
*)
Clear[ ArimaCoefBias, ArimaBiasMatrix ];

ArimaBiasMatrix[ p_, trend_ ] := Module \
	[	{mat},
		mat = DiagonalMatrix[Range[0,p]] + b2[p];
		If[trend>0,
			mat = mat + trend b3[p]];
		mat
	];

ArimaCoefficientBias[ y_ARIMA, trend_ ] := Module	\
	[	{B, phi,p},
		phi = ArimaPhi[y];
		p   = Length[phi]-1;
		B   = ArimaBiasMatrix[ p, trend ];
		Drop[B.phi,1]
	] /; ArimaIsArQ[y];


(*
:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Prediction mean squared error and infinite polynomials
:[font = text; inactive; initialization; preserveAspect; ]
Reference for effect of estimation:        Fuller and Hasza (1981)

Note the use of the global variable z in the transfer function.
:[font = input; initialization; preserveAspect; ]
*)
Clear[ ArimaMAWeights, ArimaMSE ];


ArimaMAWeights[y_ARIMA, n_Integer:10] :=	Module \
	[ 	{poly,k},
		poly = Normal[Series[
			ArimaTransferFunction[y],{z,0,n-1}]];
		Table[Coefficient[poly, z, k],{k,0,n-1}]
	]
	
ArimaMSE[y_ARIMA, n_Integer:10] := 		Module \
	[	{sqr},
		sqr = #*#& /@ ArimaMAWeights[y,n];
		(ArimaScale[y]^2) * FoldList[Plus,1,Drop[sqr,1]]
	]
	
(*
:[font = input; initialization; preserveAspect; startGroup; ]
*)
Clear[eta, buildA, powersOfA, tr]

buildA [ phi_ ] := 	Module	\
	[	{A, i, p=Length[phi]-1},
		A = Transpose[Join[	\
				IdentityMatrix[p-1],
				{Table[0,{i,p-1}]} ]];
		Join[ {-Drop[phi,1]}, A]
	]
	
powersOfA [ A_, f_]	:= Module	\
	[	{last, result, dim=Length[A]},
		last = A;
		Join[ {IdentityMatrix[dim], A},
			  Table[ last=last.A, {f-2}]]
	]

tr[ mat_ ] := Module[	{i},
	Sum[ mat[[i,i]], {i,Length[mat]}]];
	
(* Note: sums really go from 0 to f-1... *)	

eta[ phi_, w_, cov_, leads_Integer ] := With \
	[	{f, a1 = phi[[2]]},
		Table[f f a1^(2 (f-1)), {f,leads}]
	] /. Length[phi] == 2;

eta[ phi_, w_, cov_, leads_Integer ] := Module	\
	[	{A, covInv, mat, i, j, f},
		A = powersOfA[ buildA[phi], leads];
		covInv = Inverse[cov];
		Table[
			Sum[ w[[i]] w[[j]]
				tr[cov.Transpose[A[[f+1-i]]]
 					.covInv.A[[f+1-j]] ],
 				{i,1,f},{j,1,f}],
 			{f,1,leads} ]
 	]

(*
:[font = message; inactive; preserveAspect; endGroup; ]
General::spell1: 
   Possible spelling error: new symbol name "last"
     is similar to existing symbol "Last".
:[font = input; initialization; preserveAspect; endGroup; ]
*)
ArimaMSE[y_ARIMA, f_Integer:10, nObs_] := Module \
	[	{s2, phi, p, cov, wts, sqr, mse, etaVec},
		s2 = ArimaScale[y]^2;
		phi = ArimaPhi[y];
		p = Length[phi]-1;
		cov = ArimaCovarianceMatrix[ y, p];
		wts = ArimaMAWeights[y,f];
		etaVec = eta [phi, wts, cov, f];
		sqr = #*#& /@ wts;
		mse = FoldList[Plus,1,Drop[sqr,1]];
		s2 (mse + etaVec/nObs)
	]
	
(*
:[font = subsubsection; inactive; initialization; preserveAspect; startGroup; ]
Simulating normal realizations
:[font = input; initialization; preserveAspect; ]
*)
Clear[ ArimaNRealize, NormalRandom]
(*
:[font = input; initialization; preserveAspect; ]
*)
<<LinearAlgebra`Cholesky`
(*
:[font = text; inactive; preserveAspect; ]
Initilization of an arma requires the joint covariance matrix of prior y and errors if moving average terms are present.  Structure here builds the partitioned covariance matrix
		                 yy							 ye
		                 ey        ee
Assumes that the data will be y[0], y[-1], ..., y[-nY+1] and e[0], e[-1], ..., e[-nE+1]. Note that the matrix ey is lower triangular since y[t] is uncorr with e[s] for s > t.
:[font = input; preserveAspect; ]
Clear[JointCovar];

JointCovar[y_ARIMA, nY_, nE_] := Module	\
	[	{s2, yyCov, eeCov, eyCov, w, cov,i,j},
		s2 = ArimaScale[y]^2;
		yyCov = ArimaCovarianceMatrix[y,nY];
		eeCov = Table[ If[i==j, s2, 0], {i,nE}, {j,nE}];
		w = s2 ArimaMAWeights[y, nE];
		eyCov = Table[ If[i>=j, w[[1+i-j]], 0], {i,nE}, {j,nY}];
		(* Print["In bigCovar... ", eyCov//MatrixForm]; *)
		Join[ Transpose[ Join[ yyCov, eyCov ] ],
			Transpose[ Join[Transpose[eyCov],eeCov]]]
	]
:[font = text; inactive; preserveAspect; ]
Initialization divided into three cases.  Pure AR, pure MA, and mixure.  The function returns a list of two values:  initial Y's and initial E's.  No use of modules or switch via the argument list checking form with three calls.
:[font = input; initialization; preserveAspect; ]
*)
Clear[Initialize]

Initialize[y_ARIMA, nAR_, 0] :=			(* pure AR *)
	{Table[NormalRandom[1],{nAR}] .
			CholeskyDecomposition[ArimaCovarianceMatrix[y,nAR]],
	 {}}

Initialize[y_ARIMA, 0, nMA_] :=			(* pure MA *)
	{ 	{}, 
		Table[ NormalRandom[ArimaScale[y]], {nMA}]   }

Initialize[y_ARIMA, nAR_, nMA_] := 
		List[Take[#,nAR],Drop[#,nAR]]& [
			 Table[ NormalRandom[1], {nAR+nMA}] .
			 	CholeskyDecomposition[JointCovar[y,nAR,nMA]]]
(*
:[font = text; inactive; preserveAspect; ]
Generator method must be fed the appropriate starting values for the recursion.  As for initialization, generation has three methods as appropriate for AR, MA, and ARMA recursions.  Far more efficient to avoid the null operations.  Actually, a fourth method is included for white noise case.
:[font = input; initialization; preserveAspect; startGroup; ]
*)
Clear[NormalRandom, Generate]

NormalRandom[sigma_] := 	(* N[0,s] *)
	N[ sigma Sqrt[-2 Log[ Random[] ]] Cos[2Pi Random[]]];

Generate[y_, n_, initY_, _] := \
		Join[initY,
		 	 Table[NormalRandom[ArimaScale[y]], {n}]  ] \
		/; ArimaIsWhiteNoiseQ[y];
											(* AR case *)
Generate[y_, n_, initY_, {}] := Module	\
	[	{data,size,phi,p,i,const, scl},
		phi   = - Reverse[ Drop[ArimaPhi[y],1]];  (* drop the 1 from phi *)
		p = Length[phi];
		size  = Length[initY];
		(* Print["Init y", initY]; *)
		data  = Join[initY, Table[0,{n}]];
		const = ArimaConstant[y];
		scl = ArimaScale[y];
		For[i=size+1, i<=size+n, i++,
			data[[i]]= const+phi.Take[data,{i-p,i-1}]+NormalRandom[scl]];
		data
	];
											(* MA case *)
Generate[y_, n_, {}, initE_] := Module		\
	[	{e,size,q,theta,i,scl},
		theta = Reverse[ ArimaTheta[y] ];
		q = Length[theta];
		scl   = ArimaScale[y];
		size  = Length[initE];
		e     = Join[initE, Table[NormalRandom[scl],{n}]];
		mean = ArimaMean[y];
		For[i=size+1, i<=size+n, i++,
			e[[i]]= mean+theta.Take[e,{i-q+1,i}]];
		Drop[e,q-1]
	];
	
Generate[y_, n_, initY_, initE_] := Module	\
	[	{data,e,sizeY,sizeE,p,q,phi,theta,i,const,scl,diff},
		phi   = - Reverse[ Drop[ArimaPhi[y],1]];  (* drop the 1 from phi *)
		theta =   Reverse[ ArimaTheta[y] ];
		p = Length[phi]; q = Length[theta];
		scl = ArimaScale[y];
		sizeY  = Length[initY];
		sizeE  = Length[initE];
		diff = sizeY-sizeE;
		e     = Join[initE, Table[NormalRandom[scl], {n}]];
		data  = Join[initY, Table[0,{n}]];
		const = ArimaConstant[y];
		For[i=sizeY+1, i<=sizeY+n, i++,
			j = i - diff;
			data[[i]]= const+phi.Take[data,{i-p,i-1}]+theta.Take[e,{j-q+1,j}]];
		data
	];
(*
:[font = message; inactive; preserveAspect; ]
General::spell1: 
   Possible spelling error: new symbol name "sizeY"
     is similar to existing symbol "size".
:[font = message; inactive; preserveAspect; endGroup; ]
General::spell: 
   Possible spelling error: new symbol name "sizeE"
     is similar to existing symbols {size, sizeY}.
:[font = text; inactive; preserveAspect; ]
External function call is a shell that parses input options, and then calls Initialize and Generate.
;[s]
5:0,0;76,1;86,2;91,3;99,4;101,-1;
5:1,11,8,Times,0,12,0,0,0;1,10,8,Courier,1,12,0,0,0;1,11,8,Times,0,12,0,0,0;1,10,8,Courier,1,12,0,0,0;1,11,8,Times,0,12,0,0,0;
:[font = input; initialization; preserveAspect; startGroup; ]
*)
Clear[ ArimaNRealize]

Options[ArimaNRealize] = { initial -> {} };

ArimaNRealize::initial = "Too few in `1`; need `2`";
ArimaNRealize::mainit  = "Initializing disrupts joint covariance";

ArimaNRealize[y_ARIMA, n_Integer:25, opts___] := Module	\
	[	{start, initY, p, q, init,data},
		initY = initial/.{opts}/.Options[ArimaNRealize];
		p = ArimaP[y]; q = ArimaQ[y];
		If[ Length[initY] > 0 && q > 0,
			Message[ArimaNRealize::mainit]];
		If[ Length[initY] > 0 && Length[initY] < p,
			Message[ArimaNRealize::initial, initY, p];
			Return[{}]  ];
		start = If[ Length[initY] > 0,
			{initY, Table[NormalRandom[ArimaScale[y]],{q}]},
			Initialize[y,p,q]];
		(* Print["n=",n,", startY =", start[[1]], ", startE=", start[[2]] ]; *)
		data = Generate[y,n,start[[1]], start[[2]]];
		If[ Length[initY] == 0,
			Drop[data,p],
			data]
	]
(*
:[font = message; inactive; preserveAspect; endGroup; endGroup; ]
General::spell: 
   Possible spelling error: new symbol name "init"
     is similar to existing symbols {initE, initY}.
:[font = subsubsection; inactive; Cclosed; preserveAspect; startGroup; ]
Testing Code for process initialization and generation
:[font = input; preserveAspect; startGroup; ]
ArimaNRealize[ar,10]
:[font = message; inactive; preserveAspect; ]
Join::heads: 
   Heads Dot and List at positions 1 and 2
     are expected to be the same.
:[font = message; inactive; preserveAspect; ]
Dot::dotsh: Tensors {-0.5, -0.25, -0.5} and {}
     have incompatible shapes.
:[font = message; inactive; preserveAspect; ]
Part::part: Part 3 of Join[{0.0662388, 0.223032, 1.13918} . 
      <<1>>, <<1>>] does not exist.
:[font = message; inactive; preserveAspect; ]
Take::take: Cannot take positions 1 through 3 in 
    Join[{0.0662388, 0.223032, 1.13918} . <<1>>, <<1>>].
:[font = message; inactive; preserveAspect; ]
Part::part: Part 4 of Join[{0.0662388, 0.223032, 1.13918} . 
      <<1>>, <<1>>] does not exist.
:[font = message; inactive; preserveAspect; ]
Take::take: Cannot take positions 2 through 4 in 
    Join[{0.0662388, 0.223032, 1.13918} . <<1>>, <<1>>].
:[font = message; inactive; preserveAspect; ]
Part::part: Part 5 of Join[{0.0662388, 0.223032, 1.13918} . 
      <<1>>, <<1>>] does not exist.
:[font = message; inactive; preserveAspect; ]
General::stop: 
   Further output of Part::part
     will be suppressed during this calculation.
:[font = message; inactive; preserveAspect; ]
Take::take: Cannot take positions 3 through 5 in 
    Join[{0.0662388, 0.223032, 1.13918} . <<1>>, <<1>>].
:[font = message; inactive; preserveAspect; ]
General::stop: 
   Further output of Take::take
     will be suppressed during this calculation.
:[font = message; inactive; preserveAspect; ]
Drop::drop: Cannot drop positions 1 through 3 in 
    Join[{0.0662388, 0.223032, 1.13918} . <<1>>, <<1>>].
:[font = output; output; inactive; preserveAspect; endGroup; ]
Drop[Join[{0.06623875867706771, 0.2230322475407212, 
     1.139180719293826} . 
    CholeskyDecomposition[{{1.777777777777778, 
       -0.888888888888889, 0.4444444444444446}, 
      {-0.888888888888889, 1.777777777777778, 
       -0.888888888888889}, 
      {0.4444444444444446, -0.888888888888889, 
       1.777777777777778}}], {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}]\
   , 3]
;[o]
Drop[Join[{0.0662388, 0.223032, 1.13918} . 
 
    CholeskyDecomposition[{{1.77778, -0.888889, 0.444444}, 
 
      {-0.888889, 1.77778, -0.888889}, 
 
      {0.444444, -0.888889, 1.77778}}], 
 
   {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}], 3]
:[font = input; preserveAspect; ]
ArimaNRealize[ma,10]
:[font = input; preserveAspect; startGroup; ]
ArimaNRealize[arma,10]
:[font = print; inactive; preserveAspect; ]
initY is {}
ARMA(1,1) model
arlong{1., 0.5, 0}maLong{1., 0.44, 0}
n=10, startY ={-2.95482}, startE={-2.80296}
{-2.95482, 0.744338, 1.85758, 11.9305, -4.89047, -5.14725, 4.61843, 
 
  -0.247859, -5.83043, 4.80299, -0.627267}
:[font = output; output; inactive; preserveAspect; endGroup; ]
{0.7443382245374101894, 1.857582095436019526, 11.93045252528971014, 
  -4.890469577783510344, -5.14724736159383265, 4.618425054351518789, 
  -0.2478587375921685696, -5.830426734306588478, 4.802993201394301206, 
  -0.6272672563242167237}
;[o]
{0.744338, 1.85758, 11.9305, -4.89047, -5.14725, 4.61843, -0.247859, 
 
  -5.83043, 4.80299, -0.627267}
:[font = input; preserveAspect; startGroup; ]
ArimaNRealize[arma,10,initial->{1,1,1}]
:[font = print; inactive; preserveAspect; ]
initY is {1, 1, 1}
n=10, startY ={1, 1, 1}, startE={2.77402}
{1, 1, 1, 0.883774, 4.44448, 1.43721, -3.06366, 3.44657, -4.95214, 
 
  0.508258, -6.69098, 0.999617, 3.97833}
:[font = message; inactive; preserveAspect; ]
ArimaNRealize::mainit: Initializing disrupts joint covariance
:[font = output; output; inactive; preserveAspect; endGroup; endGroup; ]
{1, 1, 1, 0.8837735809895578523, 4.444482530388695246, 
  1.43721372603710239, -3.063657682099898604, 3.446568039705086475, 
  -4.952136599517259315, 0.5082582719032034767, -6.690983013221599507, 
  0.9996174190427234122, 3.978330130452931921}
;[o]
{1, 1, 1, 0.883774, 4.44448, 1.43721, -3.06366, 3.44657, -4.95214, 
 
  0.508258, -6.69098, 0.999617, 3.97833}
:[font = subsubsection; inactive; initialization; Cclosed; preserveAspect; startGroup; ]
Some Elements of Estimation
:[font = input; initialization; preserveAspect; endGroup; ]
*)
Clear[ EstimateCovariance, EstimateCorrelation ]

EstimateCovariance[data_List, nLags_Integer] := Module	\
	[	{t,j, n, dev},
		n = Length[data];
		dev = data - (Plus @@ data)/n;
		Table[
			Sum[dev[[t]]dev[[t+j]],{t,n-j}] / n,
			{j,0,nLags-1}]
	]
	
EstimateCorrelation[data_List, nLags_Integer] := Module	\
	[	{cov},
		cov = EstimateCovariance[data,nLags];
		cov / cov[[1]]
	]
(*
:[font = input; initialization; preserveAspect; ]
*)
(* 

End[]	(* private context *)

EndPackage[]

*)
(*
^*)