(* ==================================================================
The use of this Package is described in "Economic and Financial
Modeling with Mathematica", Hal Varian, editor, TELOS/Springer-Verlag,
1993, ISBN 0-387-97882-8.  $44.95 (Includes 1.44 MS-DOS disk.)  To
place order call 1-800-777-4643 or fax 1-201-348-4505.
=====================================================================*)
(*^

::[currentKernel; 
	fontset = title, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeTitle, center, M7, bold, L1, e8,  24, "Times"; ;
	fontset = subtitle, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeTitle, center, M7, bold, L1, e6,  18, "Times"; ;
	fontset = subsubtitle, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeTitle, center, M7, italic, L1, e6,  14, "Times"; ;
	fontset = section, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeSection, grayBox, M22, bold, L1, a20,  18, "Times"; ;
	fontset = subsection, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeSection, blackBox, M19, bold, L1, a15,  14, "Times"; ;
	fontset = subsubsection, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeSection, whiteBox, M18, bold, L1, a12,  12, "Times"; ;
	fontset = text, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, L1,  12, "Times"; ;
	fontset = smalltext, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, L1,  10, "Times"; ;
	fontset = input, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeInput, M42, N23, bold, L-5,  12, "Courier"; ;
	fontset = output, output, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23, L-5,  12, "Courier"; ;
	fontset = message, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23, R65535, L-5,  12, "Courier"; ;
	fontset = print, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23, L-5,  12, "Courier"; ;
	fontset = info, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23, B65535, L-5,  12, "Courier"; ;
	fontset = postscript, PostScript, formatAsPostScript, output, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeGraphics, M7, l34, w282, h287, L1,  12, "Courier"; ;
	fontset = name, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, italic, L1,  10, "Geneva"; ;
	fontset = header, inactive, noKeepOnOnePage, preserveAspect, M7, L1,  12, "Times"; ;
	fontset = Left Header, inactive,  12, "Times"; ;
	fontset = footer, inactive, noKeepOnOnePage, preserveAspect, center, M7, L1,  12, "Times"; ;
	fontset = Left Footer, inactive,  12, "Times"; ;
	fontset = help, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, L1,  10, "Times"; ;
	fontset = clipboard, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, L1,  12, "Times"; ;
	fontset = completions, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, L1,  12, "Times"; ;
	fontset = special1, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, L1,  12, "Times"; ;
	fontset = special2, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, L1,  12, "Times"; ;
	fontset = special3, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, L1,  12, "Times"; ;
	fontset = special4, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, L1,  12, "Times"; ;
	fontset = special5, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, L1,  12, "Times"; ;]
:[font = input; initialization; preserveAspect; ]
*)
(*:Version: Mathematica 2.0 *)
(*:Name:    Stochastic *)
(*:Author:  Colin Rose *)
(*:Legal:   Copyright (c) April 1992, Colin Rose *) 
(*:Summary: This package provides tools for:

            
            Deriving the mean, variance, pdf and cdf of:
      1.    Lower-truncated, upper-truncated and 
            doubly-truncated normal distributions.
      2.    Lower-censored, upper-censored and 
            doubly-censored normal distributions. *)
            
               
(*:Requirements: This package needs the package 
                 Statistics`NormalDistribution`. It will be
                 read in automatically. *)

(*:Warning: Users of Windows MAY have trouble loading
            the normal distribution package. Contact 
            Wolfram if you should be so afflicted  *)

BeginPackage["Stochastic`", {"Statistics`NormalDistribution`"}]

     
LowerPlot::useage =
        "LowerPlot plots the pdf and cdf of a lower
         truncated normal distribution, given mu, sig and sLow."

UpperPlot::useage =
        "UpperPlot plots the pdf and cdf of an upper
         truncated normal distribution, given mu, sig and sUp."

DoublePlot::useage =
        "DoublePlot plots the pdf and cdf of a doubly
         truncated normal distribution, given mu, sig, sLow and sUp."



TruncLowerMean::useage =
        "TruncLowerMean calculates the mean of a lower
         truncated normal distribution, given mu, sig and sLow."

TruncUpperMean::useage =
        "TruncUpperMean calculates the mean of an upper
         truncated normal distribution, given mu, sig and sUp."

TruncDoubleMean::useage =
        "TruncDoubleMean calculates the mean of a doubly
         truncated normal distribution, given mu, sig, sLow and sUp."

TruncLowerVAR::useage =
        "TruncLowerVAR calculates the variance of a lower
         truncated normal distribution, given mu, sig and sLow."

TruncUpperVAR::useage =
        "TruncUpperVAR calculates the variance of an upper
         truncated normal distribution, given mu, sig and sUp."

TruncDoubleVAR::useage =
        "TruncDoubleVAR calculates the variance of a doubly
         truncated normal distribution, given mu, sig, sLow and sUp."



CensLowerMean::useage =
        "CensLowerMean calculates the mean of a lower
         censored normal distribution, given mu, sig and sLow."

CensUpperMean::useage =
        "CensUpperMean calculates the mean of an upper
         censored normal distribution, given mu, sig and sUp."

CensDoubleMean::useage =
        "CensDoubleMean calculates the mean of a doubly
         censored normal distribution, given mu, sig, sLow and sUp."

CensLowerVAR::useage =
        "CensLowerVAR calculates the variance of a lower
         censored normal distribution, given mu, sig and sLow."

CensUpperVAR::useage =
        "CensUpperVAR calculates the variance of an upper
         censored normal distribution, given mu, sig and sUp."

CensDoubleVAR::useage =
        "CensDoubleVAR calculates the variance of a doubly
         censored normal distribution, given mu, sig, sLow and sUp."




g[s_] := Statistics`Common`DistributionsCommon`PDF[
         Statistics`NormalDistribution`NormalDistribution[mu, sig], s]
G[s_] := Statistics`Common`DistributionsCommon`CDF[
         Statistics`NormalDistribution`NormalDistribution[mu, sig], s]

Lg[s_] := g[s]/(1-G[sLow])
LG[s_] := (G[s]-G[sLow])/(1-G[sLow])

Ug[s_] := g[s]/G[sUp]
UG[s_] := G[s]/G[sUp]

Dg[s_] := g[s]/( G[sUp]-G[sLow] )
DG[s_] := ( G[s]-G[sLow] )/( G[sUp]-G[sLow] )


PM1 := Plot[g[s], {s, mu - 3 sig, mu + 3 sig}, DisplayFunction -> Identity]
PM2 := Plot[G[s], {s, mu - 3 sig, mu + 3 sig}, DisplayFunction -> Identity]
PM3 := Plot[Lg[s], {s, sLow, mu + 3 sig}, DisplayFunction -> Identity, PlotRange -> All, PlotStyle -> {AbsoluteThickness[1.5]}]
PM4 := Plot[LG[s], {s, sLow, mu + 3 sig}, DisplayFunction -> Identity, PlotRange -> All, PlotStyle -> {AbsoluteThickness[1.5]}]
PM5 := Plot[Ug[s], {s, mu - 3 sig, sUp}, DisplayFunction -> Identity, PlotRange -> All, PlotStyle -> {AbsoluteThickness[1.5]}]
PM6 := Plot[UG[s], {s, mu - 3 sig, sUp}, DisplayFunction -> Identity, PlotRange -> All, PlotStyle -> {AbsoluteThickness[1.5]}]
PM7 := Plot[Dg[s], {s, sLow, sUp}, DisplayFunction -> Identity, PlotRange -> All, PlotStyle -> {AbsoluteThickness[1.5]}]
PM8 := Plot[DG[s], {s, sLow, sUp}, DisplayFunction -> Identity, PlotRange -> All, PlotStyle -> {AbsoluteThickness[1.5]}]


JJ1 := Show[PM1, PM3, DisplayFunction -> Identity, PlotRange -> All, PlotLabel -> "Probability Density Function", 
       Epilog -> {AbsoluteDashing[{9, 6}], AbsoluteThickness[1.5], 
       Line[{{sLow, 0}, {sLow, Lg[sLow]}} ], 
       Line[{{mu + 3 sig,0}, {mu + 3 sig, Lg[mu + 3 sig]} }]}]
JJ2 := Show[PM2, PM4, DisplayFunction -> Identity, PlotRange -> All, PlotLabel -> "Distribution Function"]
JJ3 := Show[PM1, PM5, DisplayFunction -> Identity, PlotRange -> All, PlotLabel -> "Probability Density Function", 
       Epilog -> {AbsoluteDashing[{9, 6}], AbsoluteThickness[1.5], 
       Line[{{mu - 3 sig, 0}, {mu - 3 sig, Ug[mu - 3 sig]}} ], 
       Line[{{sUp,0}, {sUp, Ug[sUp]} }]}]
JJ4 := Show[PM2, PM6, DisplayFunction -> Identity, PlotRange -> All, PlotLabel -> "Distribution Function"]
JJ5 := Show[PM1, PM7, DisplayFunction -> Identity, PlotRange -> All, PlotLabel -> "Probability Density Function",
       Epilog -> {AbsoluteDashing[{9, 6}], AbsoluteThickness[1.5],
       Line[{{sLow, 0}, {sLow, Dg[sLow]}} ], 
       Line[{{sUp,0}, {sUp, Dg[sUp]} }]}]
JJ6 := Show[PM2, PM8, DisplayFunction -> Identity, PlotRange -> All, PlotLabel -> "Distribution Function"]


LowerPlot  := Show[GraphicsArray[{JJ1, JJ2}], DisplayFunction -> $DisplayFunction]
UpperPlot  := Show[GraphicsArray[{JJ3, JJ4}], DisplayFunction -> $DisplayFunction]
DoublePlot := Show[GraphicsArray[{JJ5, JJ6}], DisplayFunction -> $DisplayFunction]




(* Truncated Normal Distributions *)

TruncLowerMean  := mu + sig^2 g[sLow]/(1-G[sLow])
TruncUpperMean  := mu - sig^2 g[sUp]/G[sUp]
TruncDoubleMean := mu - sig^2 (g[sUp]- g[sLow])/(G[sUp]-G[sLow])


TruncLowerVAR   := sig^2(1 + (sLow-mu)g[sLow]/(1-G[sLow]) - sig^2(g[sLow]/(1-G[sLow]))^2)
TruncUpperVAR   := sig^2(1 - (sUp-mu)g[sUp]/G[sUp] - sig^2(g[sUp]/G[sUp])^2)
TruncDoubleVAR  := sig^2(1 - ((sUp-mu)g[sUp] - (sLow-mu)g[sLow])/(G[sUp]-G[sLow]) 
                    - sig^2((g[sUp]- g[sLow])/(G[sUp]-G[sLow]))^2)



(* Censored Normal Distributions *)

CensLowerMean  := mu(1-G[sLow]) + sig^2 g[sLow] + (sLow)G[sLow]
CensUpperMean  := mu(G[sUp]) - sig^2 g[sUp]  + (sUp)(1 - G[sUp])
CensDoubleMean := mu(G[sUp] - G[sLow]) - sig^2 (g[sUp] - g[sLow]) + (sLow)G[sLow] + (sUp)(1 - G[sUp])


CensLowerVAR   :=  sLow^2 G[sLow] + (mu^2 + sig^2)(1-G[sLow]) + sig^2 (sLow + mu)g[sLow] - (CensLowerMean)^2
CensUpperVAR   :=  sUp^2 (1-G[sUp]) + (mu^2 + sig^2)G[sUp] - sig^2 (sUp + mu)g[sUp] - (CensUpperMean)^2
CensDoubleVAR  :=  sLow^2 G[sLow] + sUp^2 (1-G[sUp]) + (mu^2 + sig^2)(G[sUp]-G[sLow]
                        ) - sig^2( (sUp + mu)g[sUp] - (sLow + mu)g[sLow]) - (CensDoubleMean)^2

                                    
EndPackage[]
(*
;[s]
97:0,0;6454,1;6455,0;6459,1;6462,0;6466,1;6467,0;6489,1;6491,0;6494,1;6495,0;6515,1;6516,0;6519,1;6527,0;6530,1;6532,0;6553,1;6556,0;6559,1;6565,0;6569,1;6583,0;6586,1;6592,0;6596,1;6602,0;6606,1;6609,0;6613,1;6618,0;6621,1;6629,0;6632,1;6634,0;6660,1;6665,0;6669,1;6674,0;6676,1;6692,0;6696,1;6708,0;6712,1;6715,0;6717,1;6720,0;6724,1;6728,0;6742,1;6745,0;6764,1;6765,0;6768,1;6776,0;6779,1;6785,0;6787,1;6800,0;6803,1;6814,0;6817,1;6820,0;6822,1;6825,0;6828,1;6832,0;6846,1;6849,0;6873,1;6878,0;6882,1;6886,0;6889,1;6897,0;6900,1;6906,0;6908,1;6922,0;6925,1;6929,0;6933,1;6971,0;6974,1;6977,0;6979,1;6982,0;6985,1;6990,0;6994,1;6997,0;6999,1;7002,0;7006,1;7011,0;7026,1;7029,0;7081,-1;
2:49,12,10,Courier,1,12,0,0,0;48,11,9,Courier,1,10,0,0,0;
^*)

